import os
import time

import oss2
import requests
import sqlalchemy
import xlrd
import xlwt
from dateutil.relativedelta import relativedelta
from flask import current_app, jsonify, request
from sqlalchemy import and_, or_
import zipfile

from apps.models import *

from apps.models import ProjectManagement
from apps.utils.response_code import RET
from apps.inves_manage import api_manage
from apps.util import verify_token


# 项目管理---项目列表
from apps.inves_manage.siku_view import get_num, upload_daily_picture


"""指标分析"""


# 山西省地市数据统计
@api_manage.route("/StatisticShanxiData", methods=["POST"])
def statistic_shanxi_data():
    req_dic = request.get_json()
    flag = req_dic['flag']  # 1项目签约金额完成情况, 2新开工项目计划投资额完成情况, 3固定资产投资项目资金到位完成情况
    try:
        # 获取最新的时间周期
        data_obj = ShanxiTarget.query.order_by(sqlalchemy.desc(ShanxiTarget.id)).first()
        data_time = data_obj.data_time
        # 获取最新时间周期的数据
        target_obj_li = ShanxiTarget.query.filter_by(data_time=data_time).all()
        data1 = ''
        data2 = ''
        if flag == 1:
            data = [{
                "district_name": target_obj.district_name,
                "sign_money": str(target_obj.sign_money) + '亿',
                "sign_target_money": str(target_obj.sign_target_money) + '亿',
                "sign_finish_rate": str(target_obj.sign_finish_rate) + '%',
            } for target_obj in target_obj_li]
            data1 = sorted(data, key=lambda x: x['sign_money'], reverse=True)
            data2 = sorted(data1, key=lambda x: x['sign_finish_rate'], reverse=True)
        if flag == 2:
            data = [{
                "district_name": target_obj.district_name,
                "plan_invest_money": str(target_obj.plan_invest_money) + '亿',
                "plan_target_money": str(target_obj.plan_target_money) + '亿',
                "plan_finish_rate": str(target_obj.plan_finish_rate) + '%',
            } for target_obj in target_obj_li]
            data1 = sorted(data, key=lambda x: x['plan_invest_money'], reverse=True)
            data2 = sorted(data1, key=lambda x: x['plan_finish_rate'], reverse=True)
        if flag == 3:
            data = [{
                "district_name": target_obj.district_name,
                "arrive_money": str(target_obj.arrive_money) + '亿',
                "arrive_target_money": str(target_obj.arrive_target_money) + '亿',
                "arrive_finish_rate": str(target_obj.arrive_finish_rate) + '%',
            } for target_obj in target_obj_li]
            data1 = sorted(data, key=lambda x: x['arrive_money'], reverse=True)
            data2 = sorted(data1, key=lambda x: x['arrive_finish_rate'], reverse=True)
        return jsonify(code=RET.OK, data={"data1": data1, "data2": data2}, msg="获取成功！")
    except Exception as e:
        db.session.rollback()
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="查询数据库错误！")


# 晋城市数据统计
@api_manage.route("/StatisticJcData", methods=["POST"])
def statistic_jc_data():
    req_dic = request.get_json()
    flag = req_dic['flag']
    current_year = datetime.now().year
    district_li = ["全市", "城区", "泽州县", "高平市", "阳城县", "陵川县",
                   "沁水县", "晋城经济技术开发区", "高平经济技术开发区",
                   "阳城经济技术开发区", "沁水经济技术开发区",
                   "陵川生态文化旅游示范区"]
    data_li1 = []
    data_li2 = []
    data_li3 = []
    data_li4 = []
    data_li5 = []
    data_li6 = []


    try:
        # 一。项目签约完成率(签约项目完成率：完成率（%）：签约金额（亿元）/ 年度目标（亿元）)
        for district in district_li:
            project_obj_list = ProjectManagement.query.filter(
                ProjectManagement.sign_time.like('%{}%'.format(str(current_year))),
                ProjectManagement.district.like('%{}%'.format(str(district)))
            ).all()
            if not project_obj_list:
                investment_volume = 0
            else:
                investment_volume = sum([project_obj.investment_volume for project_obj in project_obj_list])
            # 年度目标
            jc_obj = JcTarget.query.filter(
                JcTarget.district_name == district,
                JcTarget.year.like('%{}%'.format(str(current_year)))
            ).first()
            if not jc_obj:
                year_target = 0
            else:
                year_target = jc_obj.money_sign

            data = {
                "district": district,
                "rate": investment_volume / year_target if year_target and year_target != 0 else 0
            }
            data_li1.append(data)
            continue
        # 项目签约情况得分=完成率（%）*赋分（15 上传后获取）如果小于赋分（15 上传后获取），
        # 则输出完成率（%）*赋分（15 上传后获取）的结果，否则输出赋分（15）
        grade_li1 = []
        for data in data_li1:
            grade = data["rate"] * 15
            if grade < 15:
                grade_li1.append({"district": data["district"], "grade": grade})
            else:
                grade_li1.append({"district": data["district"], "grade": 15})

        # 二。项目开工率(开工率进度：当年签约当年开工个数/签约个数)
        for district in district_li:
            # 签约项目个数
            sign_num = ProjectManagement.query.filter(
                ProjectManagement.sign_time.like('%{}%'.format(str(current_year))),
                ProjectManagement.district.like('%{}%'.format(str(district)))
            ).count()
            # 年度目标
            start_num = ProjectManagement.query.filter(
                ProjectManagement.start_time.like('%{}%'.format(str(current_year))),
                ProjectManagement.district.like('%{}%'.format(str(district)))
            ).count()

            data = {
                "district": district,
                "rate": start_num / sign_num if sign_num and sign_num != 0 else 0
            }
            data_li2.append(data)
            continue
        # 项目开工率情况得分=如果开工率进度小于0.4，则输出开工率进度*赋分（5分）/0.4，否则输出赋分（5分）；
        grade_li2 = []
        for data in data_li2:
            grade = data["rate"]
            if grade < 0.4:
                grade_li2.append({"district": data["district"], "grade": grade * 5 / 0.4})
            else:
                grade_li2.append({"district": data["district"], "grade": 5})

        # 三。新开工项目计划投资额完成率得分=(完成率（%）：新开工项目计划投资额（亿元）/ 年度目标（亿元）)
        for district in district_li:
            project_obj_list = ProjectManagement.query.filter(
                ProjectManagement.start_time.like('%{}%'.format(str(current_year))),
                ProjectManagement.district.like('%{}%'.format(str(district)))
            ).all()
            if not project_obj_list:
                investment_volume = 0
            else:
                investment_volume = sum([project_obj.investment_volume for project_obj in project_obj_list])
            # 年度目标
            jc_obj = JcTarget.query.filter(
                JcTarget.district_name == district,
                JcTarget.year.like('%{}%'.format(str(current_year)))
            ).first()
            if not jc_obj:
                year_target = 0
            else:
                year_target = jc_obj.money_plan_invest

            data = {
                "district": district,
                "rate": investment_volume / year_target if year_target and year_target != 0 else 0
            }
            data_li3.append(data)
            continue
            # return jsonify(code=RET.OK, data=data_li, msg="新开工项目计划投资额完成率获取成功")
        # 新开工固定资产投资项目计划投资额完成情况=如果完成率（%）小于1，
        # 则输出完成率（%）*赋分（25 上传后获取）的结果，否则输出赋分（25）；
        grade_li3 = []
        for data in data_li3:
            grade = data["rate"]
            if grade < 1:
                grade_li3.append({"district": data["district"], "grade": grade * 25})
            else:
                grade_li3.append({"district": data["district"], "grade": 25})

        # 四。固定资产投资项目到位资金完成率(完成率（%）：固定资产投资项目资金到位额（亿元）/ 年度目标（亿元）)
        for district in district_li:
            project_obj = MoneyArrive.query.filter(
                MoneyArrive.data_time.like('%{}%'.format(str(current_year))),
                MoneyArrive.district_name.like('%{}%'.format(str(district)))
            ).first()
            if not project_obj:
                arrive_money = 0
            else:
                arrive_money = project_obj.arrive_money
            # 年度目标
            jc_obj = JcTarget.query.filter(
                JcTarget.district_name == district,
                JcTarget.year.like('%{}%'.format(str(current_year)))
            ).first()
            if not jc_obj:
                year_target = 0
            else:
                year_target = jc_obj.money_arrive_target1

            data = {
                "district": district,
                "rate": arrive_money / year_target if year_target and year_target != 0 else 0
            }
            data_li4.append(data)
            continue
            # return jsonify(code=RET.OK, data=data_li, msg="固定资产投资项目到位资金完成率获取成功")
        # 固定资产投资项目资金到位情况得分=如果完成率（%）小于1，
        # 则输出完成率（%）*赋分（25 上传后获取）的结果，否则输出赋分（25）；
        grade_li4 = []
        for data in data_li4:
            grade = data["rate"]
            if grade < 1:
                grade_li4.append({"district": data["district"], "grade": grade * 25})
            else:
                grade_li4.append({"district": data["district"], "grade": 25})

        # 五。非固定资产投资项目到位资金完成率(完成率（%）：非固定资产投资项目资金到位额（亿元）/ 年度目标（亿元）)
        for district in district_li:
            project_obj_list = ProjectManagement.query.filter(
                ProjectManagement.is_fixed_investment == '否',
                ProjectManagement.sign_time.like('%{}%'.format(str(current_year))),
                ProjectManagement.district.like('%{}%'.format(str(district)))
            ).all()
            if not project_obj_list:
                investment_volume = 0
            else:
                investment_volume = sum([project_obj.investment_year for project_obj in project_obj_list])
            # 年度目标
            jc_obj = JcTarget.query.filter(
                JcTarget.district_name == district,
                JcTarget.year.like('%{}%'.format(str(current_year)))
            ).first()
            if not jc_obj:
                year_target = 0
            else:
                year_target = jc_obj.money_arrive_target0
            data = {
                "district": district,
                "rate": investment_volume / year_target if year_target and year_target != 0 else 0
            }
            data_li5.append(data)
            continue
            # return jsonify(code=RET.OK, data=data_li, msg="固定资产投资项目到位资金完成率获取成功")
            # 综合得分
            # 项目签约情况得分
            # 得分（满分15分）：完成率（ % ）*赋分（15
            # 上传后获取）如果小于赋分（15
            # 上传后获取），则输出完成率（ % ）*赋分（15
            # 上传后获取）的结果，否则输出赋分（15）
            pass
        # 非固定资产投资项目资金到位情况得分=如果完成率（%）小于1，
        # 则输出完成率（%）*赋分（5 上传后获取）的结果，否则输出赋分（5）；
        grade_li5 = []
        for data in data_li5:
            grade = data["rate"]
            if grade < 1:
                grade_li5.append({"district": data["district"], "grade": grade * 5})
            else:
                grade_li5.append({"district": data["district"], "grade": 5})

        # 六。综合得分
        for district in district_li:
            # 签约情况得分
            for grade_dic in grade_li1:
                if grade_dic["district"] == district:
                    grade1 = grade_dic["grade"]
            # 项目开工率情况得分
            for grade_dic in grade_li2:
                if grade_dic["district"] == district:
                    grade2 = grade_dic["grade"]
            # 新开工得分
            for grade_dic in grade_li3:
                if grade_dic["district"] == district:
                    grade4 = grade_dic["grade"]
            # 固定资产得分
            for grade_dic in grade_li4:
                if grade_dic["district"] == district:
                    grade4 = grade_dic["grade"]
            # 非固定资产得分
            for grade_dic in grade_li5:
                if grade_dic["district"] == district:
                    grade5 = grade_dic["grade"]
            total_grade = grade1 + grade2 + grade4 + grade4 + grade5
            data_li6.append({"district": district, "total_grade": total_grade})
            continue
        return jsonify(code=RET.OK,
                       data={"data_li1": data_li1, "data_li2": data_li2,
                             "data_li3": data_li3, "data_li4": data_li4,
                             "data_li5": data_li5, "data_li6": data_li6},
                       msg="数据查询成功")
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="数据查询错误！")


"""项目分析"""


# 项目来源区域分布饼状图
@api_manage.route("/ArealDistribution", methods=["POST"])
def areal_distribution():
    req_dic = request.get_json()
    district = req_dic['district']
    try:
        project_obj_list = ProjectManagement.query.filter(
            ProjectManagement.district == district)
        num1 = 0  # 京津冀
        num2 = 0  # 长三角
        num3 = 0  # 珠三角
        num4 = 0  # 中原经济区
        num5 = 0  # 其他
        for project_obj in project_obj_list:
            if project_obj.project_source in ["北京", "天津", "河北"]:
                num1 += 1
            elif project_obj.project_source in ["上海", "江苏", "浙江"]:
                num2 += 1
            elif project_obj.project_source in ["广东", "香港", "澳门"]:
                num3 += 1
            elif project_obj.project_source in ["河南", "天津", "河北", "长治", "晋城", "运城",
                                          "聊城", "菏泽", "泰安", "宿州", "淮北", "阜阳",
                                          "亳州", "蚌埠", "淮南", "邢台", "邯郸",]:
                num4 += 1
            else:
                num5 += 1
        total_num = sum([num1, num2, num3, num4, num5])
        data = {
            "data1": str(round(num1 / total_num, 2)) + '%',
            "data2": str(round(num2 / total_num, 2)) + '%',
            "data3": str(round(num3 / total_num, 2)) + '%',
            "data4": str(round(num4 / total_num, 2)) + '%',
            "data5": str(round(num5 / total_num, 2)) + '%',
        }
        return jsonify(code=RET.OK, data=data, msg='数据查询成功')
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="数据查询错误！")


# 项目来源区域分布项目列表
@api_manage.route("/ArealProject", methods=["POST"])
def areal_project():
    req_dic = request.get_json()
    district = req_dic['district']
    project_source = req_dic['project_source']

    try:
        project_obj = ProjectManagement.query.filter(
            ProjectManagement.district == district,
        )
        if project_source == '京津冀':
            project_obj = ProjectManagement.filter(or_(
                ProjectManagement.province.like('%北京%'),
                ProjectManagement.province.like('%天津%'),
                ProjectManagement.province.like('%河北%')
            ))
        elif project_source == '长三角':
            project_obj = ProjectManagement.filter(or_(
                ProjectManagement.province.like('%江苏%'),
                ProjectManagement.province.like('%浙江%'),
                ProjectManagement.province.like('%上海%')
            ))
        elif project_source == '珠三角':
            project_obj = ProjectManagement.filter(or_(
                ProjectManagement.province.like('%广东%'),
                ProjectManagement.province.like('%香港%'),
                ProjectManagement.province.like('%澳门%')
            ))
        elif project_source == '中原经济区':
            project_obj = ProjectManagement.filter(or_(
                ProjectManagement.province.like('%河南%'),
                ProjectManagement.province.like('%天津%'),
                ProjectManagement.province.like('%河北%'),
                ProjectManagement.province.like('%长治%'),
                ProjectManagement.province.like('%晋城%'),
                ProjectManagement.province.like('%运城%'),
                ProjectManagement.province.like('%聊城%'),
                ProjectManagement.province.like('%菏泽%'),
                ProjectManagement.province.like('%泰安%'),
                ProjectManagement.province.like('%宿州%'),
                ProjectManagement.province.like('%淮北%'),
                ProjectManagement.province.like('%阜阳%'),
                ProjectManagement.province.like('%亳州%'),
                ProjectManagement.province.like('%蚌埠%'),
                ProjectManagement.province.like('%淮南%'),
                ProjectManagement.province.like('%邢台%'),
                ProjectManagement.province.like('%邯郸%'),
            ))
        elif project_source == '其他':
            project_obj = ProjectManagement.filter(and_(
                ProjectManagement.province.notlike('%河南%'),
                ProjectManagement.province.notlike('%天津%'),
                ProjectManagement.province.notlike('%河北%'),
                ProjectManagement.province.notlike('%长治%'),
                ProjectManagement.province.notlike('%晋城%'),
                ProjectManagement.province.notlike('%运城%'),
                ProjectManagement.province.notlike('%聊城%'),
                ProjectManagement.province.notlike('%菏泽%'),
                ProjectManagement.province.notlike('%泰安%'),
                ProjectManagement.province.notlike('%宿州%'),
                ProjectManagement.province.notlike('%淮北%'),
                ProjectManagement.province.notlike('%阜阳%'),
                ProjectManagement.province.notlike('%亳州%'),
                ProjectManagement.province.notlike('%蚌埠%'),
                ProjectManagement.province.notlike('%淮南%'),
                ProjectManagement.province.notlike('%邢台%'),
                ProjectManagement.province.notlike('%邯郸%'),

                ProjectManagement.province.notlike('%北京%'),
                ProjectManagement.province.notlike('%天津%'),
                ProjectManagement.province.notlike('%河北%'),
                ProjectManagement.province.notlike('%江苏%'),
                ProjectManagement.province.notlike('%浙江%'),
                ProjectManagement.province.notlike('%上海%'),
                ProjectManagement.province.notlike('%广东%'),
                ProjectManagement.province.notlike('%香港%'),
                ProjectManagement.province.notlike('%澳门%')
            ))
        else:
            return jsonify(code=RET.PARAMERR, msg="参数错误！")
        project_obj_list = project_obj.all()
        pass
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="数据查询错误！")
